cdef extern from "../lib/marisa/key.h" namespace "marisa" nogil:

    cdef cppclass Key:
        Key()
        Key(Key &query)

        #Key &operator=(Key &query)

        char operator[](int i)

        void set_str(char *str)
        void set_str(char *ptr, int length)
        void set_id(int id)
        void set_weight(float weight)

        char *ptr()
        int length()
        int id()
        float weight()

        void clear()
        void swap(Key &rhs)


cimport agent
cimport base
cimport keyset


cdef extern from "../lib/marisa/trie.h" namespace "marisa" nogil:

    cdef cppclass Trie:
        Trie()

        void build(keyset.Keyset &keyset, int config_flags) except +
        void build(keyset.Keyset &keyset) except +

        void mmap(char *filename) except +
        void map(void *ptr, int size) except +

        void load(char *filename) except +
        void read(int fd) except +

        void save(char *filename) except +
        void write(int fd) except +

        bint lookup(agent.Agent &agent) except +
        void reverse_lookup(agent.Agent &agent) except +KeyError
        bint common_prefix_search(agent.Agent &agent) except +
        bint predictive_search(agent.Agent &agent) except +

        int num_tries() except +
        int num_keys() except +
        int num_nodes() except +

        base.TailMode tail_mode()
        base.NodeOrder node_order()

        bint empty() except +
        int size() except +
        int total_size() except +
        int io_size() except +

        void clear() except +
        void swap(Trie &rhs) except +



cimport query, key

cdef extern from "../lib/marisa/keyset.h" namespace "marisa" nogil:
    cdef cppclass Keyset:

#        cdef enum constants:
#            BASE_BLOCK_SIZE  = 4096
#            EXTRA_BLOCK_SIZE = 1024
#            KEY_BLOCK_SIZE   = 256

        Keyset()

        void push_back(key.Key &key)
        void push_back(key.Key &key, char end_marker)

        void push_back(char *str)
        void push_back(char *ptr, int length)
        void push_back(char *ptr, int length, float weight)

        key.Key &operator[](int i)

        int num_keys()
        bint empty()

        int size()
        int total_length()

        void reset()
        void clear()
        void swap(Keyset &rhs)



from libcpp.string cimport string

cdef extern from "<istream>" namespace "std" nogil:
    cdef cppclass istream:
        istream() except +
        istream& read (char* s, int n) except +

    cdef cppclass ostream:
        ostream() except +
        ostream& write (char* s, int n) except +

cdef extern from "<sstream>" namespace "std" nogil:

    cdef cppclass stringstream:
        stringstream()
        stringstream(string s)
        string str ()




cimport query, key

cdef extern from "../lib/marisa/agent.h" namespace "marisa" nogil:
    cdef cppclass Agent:
        Agent() except +

        query.Query &query()
        key.Key &key()

        void set_query(char *str)
        void set_query(char *ptr, int length)
        void set_query(int key_id)

        void set_key(char *str)
        void set_key(char *ptr, int length)
        void set_key(int id)

        void clear()

        void init_state()

        void swap(Agent &rhs)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals, division
import random
import string
import timeit
import os
import zipfile
import struct
#import pstats
#import cProfile

import marisa_trie

def words100k():
    zip_name = os.path.join(
        os.path.abspath(os.path.dirname(__file__)),
        'words100k.txt.zip'
    )
    zf = zipfile.ZipFile(zip_name)
    txt = zf.open(zf.namelist()[0]).read().decode('utf8')
    return txt.splitlines()

def random_words(num):
    russian = 'абвгдеёжзиклмнопрстуфхцчъыьэюя'
    alphabet = '%s%s' % (russian, string.ascii_letters)
    return [
        "".join([random.choice(alphabet) for x in range(random.randint(1,15))])
        for y in range(num)
    ]

def truncated_words(words):
    return [word[:3] for word in words]

def prefixes1k(words, prefix_len):
    words = [w for w in words if len(w) >= prefix_len]
    every_nth = int(len(words)/1000)
    _words = [w[:prefix_len] for w in words[::every_nth]]
    return _words[:1000]

WORDS100k = words100k()
MIXED_WORDS100k = truncated_words(WORDS100k)
NON_WORDS100k = random_words(100000)
PREFIXES_3_1k = prefixes1k(WORDS100k, 3)
PREFIXES_5_1k = prefixes1k(WORDS100k, 5)
PREFIXES_8_1k = prefixes1k(WORDS100k, 8)
PREFIXES_15_1k = prefixes1k(WORDS100k, 15)


def format_result(key, value, text_width):
    key = key.ljust(text_width)
    print("    %s %s" % (key, value))

def bench(name, timer, descr='M ops/sec', op_count=0.1, repeats=3, runs=5,
          text_width=33):
    try:
        times = []
        for x in range(runs):
            times.append(timer.timeit(repeats))

        def op_time(time):
            return op_count*repeats / time

        val = "%0.3f%s" % (op_time(min(times)), descr)
        format_result(name, val, text_width)
    except (AttributeError, TypeError):
        format_result(name, "not supported", text_width)

def create_trie():
    words = words100k()
    return marisa_trie.Trie(words)

def create_bytes_trie():
    words = words100k()
    values = [struct.pack(str('<H'), len(word)) for word in words]
    return marisa_trie.BytesTrie(zip(words, values))

def create_record_trie():
    words = words100k()
    values = [ [len(word)] for word in words]
    return marisa_trie.RecordTrie(str('<H'), zip(words, values))


def benchmark():
    print('\n====== Benchmarks (100k unique unicode words) =======\n')

    common_setup = """
from __main__ import create_trie, create_bytes_trie, create_record_trie
from __main__ import WORDS100k, NON_WORDS100k, MIXED_WORDS100k
from __main__ import PREFIXES_3_1k, PREFIXES_5_1k, PREFIXES_8_1k, PREFIXES_15_1k
NON_WORDS_10k = NON_WORDS100k[:10000]
NON_WORDS_1k = ['ыва', 'xyz', 'соы', 'Axx', 'avы']*200
"""
    BUILD = {
        'dict': 'data = dict((word, len(word)) for word in WORDS100k);',
        'Trie': 'data = create_trie();',
        'BytesTrie': 'data = create_bytes_trie();',
        'RecordTrie': 'data = create_record_trie();',
    }

    dict_setup = common_setup + BUILD['dict']
    trie_setup = common_setup + BUILD['Trie']
    bytes_trie_setup = common_setup + BUILD['BytesTrie']
    record_trie_setup = common_setup + BUILD['RecordTrie']

    structures = [
        ('dict', dict_setup),
        ('Trie', trie_setup),
        ('BytesTrie', bytes_trie_setup),
        ('RecordTrie', record_trie_setup),
    ]

    # build performance
    for name, setup in structures:
        timer = timeit.Timer(BUILD[name], common_setup)
        bench("%s building" % name, timer, 'M words/sec', 0.1, 5)

    # common operations speed
    tests = [
        ('__getitem__ (hits)', "for word in WORDS100k: data[word]", 'M ops/sec', 0.1, 3),
        ('get() (hits)', "for word in WORDS100k: data.get(word)", 'M ops/sec', 0.1, 3),
        ('get() (misses)', "for word in NON_WORDS_10k: data.get(word)", 'M ops/sec', 0.01, 5),
        ('__contains__ (hits)', "for word in WORDS100k: word in data", 'M ops/sec', 0.1, 3),
        ('__contains__ (misses)', "for word in NON_WORDS100k: word in data", 'M ops/sec', 0.1, 3),
        ('items()', 'list(data.items())', ' ops/sec', 1, 1),
        ('keys()', 'list(data.keys())', ' ops/sec', 1, 1),
#        ('values()', 'list(data.values())', ' ops/sec', 1, 1),
    ]

    for test_name, test, descr, op_count, repeats in tests:
        for name, setup in structures:
            timer = timeit.Timer(test, setup)
            full_test_name = "%s %s" % (name, test_name)
            bench(full_test_name, timer, descr, op_count, repeats, 9)


    # trie-specific benchmarks
    for struct_name, setup in structures[1:]:
        _bench_data = [
            ('hits', 'WORDS100k'),
            ('mixed', 'MIXED_WORDS100k'),
            ('misses', 'NON_WORDS100k'),
        ]

        for meth in ['prefixes', 'iter_prefixes']:
            for name, data in _bench_data:
                bench(
                    '%s.%s (%s)' % (struct_name, meth, name),
                    timeit.Timer(
                        "for word in %s:\n"
                        "   for it in data.%s(word): pass" % (data, meth),
                        setup
                    ),
                    runs=3
                )

        _bench_data = [
            ('xxx', 'avg_len(res)==415', 'PREFIXES_3_1k'),
            ('xxxxx', 'avg_len(res)==17', 'PREFIXES_5_1k'),
            ('xxxxxxxx', 'avg_len(res)==3', 'PREFIXES_8_1k'),
            ('xxxxx..xx', 'avg_len(res)==1.4', 'PREFIXES_15_1k'),
            ('xxx', 'NON_EXISTING', 'NON_WORDS_1k'),
        ]
        for xxx, avg, data in _bench_data:
            for meth in ['keys']: #('items', 'keys', 'values'):
                bench(
                    '%s.%s(prefix="%s"), %s' % (struct_name, meth, xxx, avg),
                    timeit.Timer(
                        "for word in %s: data.%s(word)" % (data, meth),
                        setup
                    ),
                    'K ops/sec',
                    op_count=1,
                    runs=3,
                    text_width=60,
                )

def check_trie(trie, words):
    value = 0
    for word in words:
        value += trie[word]
    if value != len(words):
        raise Exception()

def profiling():
    import pstats
    import cProfile
    print('\n====== Profiling =======\n')
    trie = create_trie()
    WORDS = words100k()

#    def check_prefixes(trie, words):
#        for word in words:
#            trie.keys(word)
#    cProfile.runctx("check_prefixes(trie, NON_WORDS_1k)", globals(), locals(), "Profile.prof")
#
    cProfile.runctx("check_trie(trie, WORDS)", globals(), locals(), "Profile.prof")

    s = pstats.Stats("Profile.prof")
    s.strip_dirs().sort_stats("time").print_stats(20)


if __name__ == '__main__':
    benchmark()
    #profiling()
    print('\n~~~~~~~~~~~~~~\n')



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import tempfile
import pickle

import pytest

import marisa_trie
from .utils import get_random_words


def test_build():
    keys = get_random_words(1000)
    trie = marisa_trie.Trie(keys)

    for key in keys:
        assert key in trie

    non_key = '2135'
    assert non_key not in trie


def test_contains():
    trie = marisa_trie.Trie()
    assert 'foo' not in trie

    trie = marisa_trie.Trie(['foo'])
    assert 'foo' in trie
    assert 'f' not in trie


def test_key_id():
    words = ['foo', 'bar', 'f']
    trie = marisa_trie.Trie(words)
    for word in words:
        key_id = trie.key_id(word)
        assert trie.restore_key(key_id) == word

    key_ids = [trie.key_id(word) for word in words]
    non_existing_id = max(key_ids) + 1

    with pytest.raises(KeyError):
        trie.restore_key(non_existing_id)

    with pytest.raises(KeyError):
        print(trie.key_id('fo'))


def test_getitem():
    words = ['foo', 'bar', 'f']
    trie = marisa_trie.Trie(words)
    for word in words:
        key_id = trie[word]
        assert trie.restore_key(key_id) == word

    key_ids = [trie[word] for word in words]
    non_existing_id = max(key_ids) + 1

    with pytest.raises(KeyError):
        trie.restore_key(non_existing_id)

    with pytest.raises(KeyError):
        print(trie['fo'])


def test_get():
    words = ['foo', 'bar', 'f']
    trie = marisa_trie.Trie(words)
    for word in words:
        key_id = trie.get(word)
        assert trie.restore_key(key_id) == word

        key_id = trie.get(word.encode('utf8'))
        assert trie.restore_key(key_id) == word

        key_id = trie.get(word, 'default value')
        assert trie.restore_key(key_id) == word

    assert trie.get('non_existing_key') is None
    assert trie.get(b'non_existing_bytes_key') is None
    assert trie.get('non_existing_key', 'default value') == 'default value'
    assert trie.get(b'non_existing_bytes_key', 'default value') == 'default value'


def test_saveload():
    fd, fname = tempfile.mkstemp()

    words = ['foo', 'bar', 'f']
    trie = marisa_trie.Trie(words)

    with open(fname, 'w') as f:
        trie.write(f)

    with open(fname, 'r') as f:
        trie2 = marisa_trie.Trie()
        trie2.read(f)

    for word in words:
        assert word in trie2


def test_mmap():
    fd, fname = tempfile.mkstemp()
    words = get_random_words(1000)
    trie = marisa_trie.Trie(words)
    with open(fname, 'w') as f:
        trie.write(f)

    trie2 = marisa_trie.Trie()
    trie2.mmap(fname)

    for word in words:
        assert word in trie2


def test_dumps_loads():
    words = get_random_words(1000)
    trie = marisa_trie.Trie(words)
    data = trie.tobytes()

    trie2 = marisa_trie.Trie().frombytes(data)

    for word in words:
        assert word in trie2
        assert trie2.key_id(word) == trie.key_id(word)


def test_pickling():
    words = get_random_words(1000)
    trie = marisa_trie.Trie(words)

    data = pickle.dumps(trie)
    trie2 = pickle.loads(data)

    for word in words:
        assert word in trie2
        assert trie2.key_id(word) == trie.key_id(word)


def test_cmp():
    trie = marisa_trie.Trie()
    assert trie == trie
    assert trie == marisa_trie.Trie()

    trie = marisa_trie.Trie(["foo", "bar"])
    assert trie == marisa_trie.Trie(["foo", "bar"])
    assert trie != marisa_trie.Trie(["foo", "boo"])

    lo_trie = marisa_trie.Trie(order=marisa_trie.LABEL_ORDER)
    wo_trie = marisa_trie.Trie(order=marisa_trie.WEIGHT_ORDER)
    assert lo_trie == lo_trie and wo_trie == wo_trie
    assert lo_trie != wo_trie

    with pytest.raises(TypeError):
        marisa_trie.Trie() < marisa_trie.Trie()

    with pytest.raises(TypeError):
        marisa_trie.Trie() > marisa_trie.Trie()

    # not sure if it makes sense copy-pasting further.


def test_iter():
    trie = marisa_trie.Trie(["foo", "bar"])
    assert list(trie) == list(trie.iterkeys())


def test_len():
    trie = marisa_trie.Trie()
    assert len(trie) == 0

    trie = marisa_trie.Trie(['foo', 'f', 'bar'])
    assert len(trie) == 3


def test_prefixes():
    trie = marisa_trie.Trie(['foo', 'f', 'foobar', 'bar'])
    assert trie.prefixes('foobar') == ['f', 'foo', 'foobar']
    assert trie.prefixes('foo') == ['f', 'foo']
    assert trie.prefixes('bar') == ['bar']
    assert trie.prefixes('b') == []

    assert list(trie.iter_prefixes('foobar')) == ['f', 'foo', 'foobar']


def test_keys():
    keys = ['foo', 'f', 'foobar', 'bar']
    trie = marisa_trie.Trie(keys)
    assert set(trie.keys()) == set(keys)


def test_keys_prefix():
    keys = ['foo', 'f', 'foobar', 'bar']
    trie = marisa_trie.Trie(keys)
    assert set(trie.keys('fo')) == set(['foo', 'foobar'])
    assert trie.keys('foobarz') == []


def test_iterkeys():
    keys = get_random_words(1000)
    trie = marisa_trie.Trie(keys)
    assert trie.keys() == list(trie.iterkeys())

    for key in keys:
        prefix = key[:5]
        assert trie.keys(prefix) == list(trie.iterkeys(prefix))


def test_items():
    keys = ['foo', 'f', 'foobar', 'bar']
    trie = marisa_trie.Trie(keys)
    items = trie.items()
    assert set(items) == set(zip(keys, (trie[k] for k in keys)))


def test_items_prefix():
    keys = ['foo', 'f', 'foobar', 'bar']
    trie = marisa_trie.Trie(keys)
    assert set(trie.items('fo')) == set([
        ('foo', trie['foo']),
        ('foobar', trie['foobar']),
    ])


def test_iteritems():
    keys = get_random_words(1000)
    trie = marisa_trie.Trie(keys)
    assert trie.items() == list(trie.iteritems())

    for key in keys:
        prefix = key[:5]
        assert trie.items(prefix) == list(trie.iteritems(prefix))


def test_has_keys_with_prefix():
    empty_trie = marisa_trie.Trie()
    assert empty_trie.has_keys_with_prefix('') == False
    assert empty_trie.has_keys_with_prefix('ab') == False

    fruit_trie = marisa_trie.Trie(['apple', 'pear', 'peach'])
    assert fruit_trie.has_keys_with_prefix('') == True
    assert fruit_trie.has_keys_with_prefix('a') == True
    assert fruit_trie.has_keys_with_prefix('pe') == True
    assert fruit_trie.has_keys_with_prefix('pear') == True
    assert fruit_trie.has_keys_with_prefix('x') == False


def test_invalid_file():
    try:
        marisa_trie.Trie().load(__file__)
    except RuntimeError as e:
        assert "MARISA_FORMAT_ERROR" in e.args[0]
    else:
        raise AssertionError("Exception is not raised")


#def test_int_trie():
#    data = {'foo': 10, 'f': 20, 'foobar':30, 'bar': 40}
#    trie = marisa_trie.IntTrie().build(data)
#
#    for key, value in data.items():
#        assert trie[key] == value
#
#    trie['foo'] = 500
#    assert trie['foo'] == 500
#
#    # trie is frozen, no new keys are allowed
#    with pytest.raises(KeyError):
#        trie['z'] = 200



cdef extern from "../lib/marisa/query.h" namespace "marisa" nogil:

    cdef cppclass Query:
        Query()
        Query(Query &query)

        #Query &operator=(Query &query)

        char operator[](int i)

        void set_str(char *str)
        void set_str(char *ptr, int length)
        void set_id(int id)

        char *ptr()
        int length()
        int id()

        void clear()
        void swap(Query &rhs)


# cython: profile=False
from __future__ import unicode_literals

from std_iostream cimport stringstream, istream, ostream
from libc.string cimport strncmp
cimport keyset
cimport key
cimport query
cimport agent
cimport trie
cimport iostream
cimport base

import itertools
import struct

try:
    from itertools import izip
except ImportError:
    izip = zip


DEFAULT_CACHE = base.MARISA_DEFAULT_CACHE
HUGE_CACHE = base.MARISA_HUGE_CACHE
LARGE_CACHE = base.MARISA_LARGE_CACHE
NORMAL_CACHE = base.MARISA_NORMAL_CACHE
SMALL_CACHE = base.MARISA_SMALL_CACHE
TINY_CACHE = base.MARISA_TINY_CACHE

MIN_NUM_TRIES = base.MARISA_MIN_NUM_TRIES
MAX_NUM_TRIES = base.MARISA_MAX_NUM_TRIES
DEFAULT_NUM_TRIES = base.MARISA_DEFAULT_NUM_TRIES

# MARISA_TEXT_TAIL merges last labels as zero-terminated strings. So, it is
# available if and only if the last labels do not contain a NULL character.
# If MARISA_TEXT_TAIL is specified and a NULL character exists in the last
# labels, the setting is automatically switched to MARISA_BINARY_TAIL.
TEXT_TAIL = base.MARISA_TEXT_TAIL

# MARISA_BINARY_TAIL also merges last labels but as byte sequences. It uses
# a bit vector to detect the end of a sequence, instead of NULL characters.
# So, MARISA_BINARY_TAIL requires a larger space if the average length of
# labels is greater than 8.
BINARY_TAIL = base.MARISA_BINARY_TAIL
DEFAULT_TAIL = base.MARISA_DEFAULT_TAIL


# MARISA_LABEL_ORDER arranges nodes in ascending label order.
# MARISA_LABEL_ORDER is useful if an application needs to predict keys in
# label order.
LABEL_ORDER = base.MARISA_LABEL_ORDER

# MARISA_WEIGHT_ORDER arranges nodes in descending weight order.
# MARISA_WEIGHT_ORDER is generally a better choice because it enables faster
# matching.
WEIGHT_ORDER = base.MARISA_WEIGHT_ORDER
DEFAULT_ORDER = base.MARISA_DEFAULT_ORDER

cdef unicode _get_key(agent.Agent& ag):
    return <unicode>(ag.key().ptr()[:ag.key().length()].decode('utf8'))


cdef class _Trie:
    """
    Base MARISA-trie wrapper.
    It can store unicode keys and assign an unque ID to each key.
    """

    cdef trie.Trie* _trie

    def __init__(self, arg=None, num_tries=DEFAULT_NUM_TRIES, binary=False,
                       cache_size=DEFAULT_CACHE, order=DEFAULT_ORDER,
                       weights=None):
        """
        ``arg`` can be one of the following:

        * an iterable with unicode keys;
        * None (if you're going to load a trie later).

        Pass a ``weights`` iterable with expected lookup frequences
        to optimize lookup and prefix search speed.
        """

        if self._trie:
            return
        self._trie = new trie.Trie()

        byte_keys = (key.encode('utf8') for key in (arg or []))

        self._build(
            byte_keys,
            weights,
            num_tries=num_tries,
            binary=binary,
            cache_size=cache_size,
            order=order
        )

    def __dealloc__(self):
        if self._trie:
            del self._trie

    def _config_flags(self, num_tries=DEFAULT_NUM_TRIES, binary=False,
                            cache_size=DEFAULT_CACHE, order=DEFAULT_ORDER):

        if not MIN_NUM_TRIES <= num_tries <= MAX_NUM_TRIES:
            raise ValueError("num_tries (which is %d) must be between between %d and %d" % (num_tries, MIN_NUM_TRIES, MAX_NUM_TRIES))

        binary_flag = BINARY_TAIL if binary else TEXT_TAIL
        return num_tries | binary_flag | cache_size | order

    def _build(self, byte_keys, weights=None, **options):
        """
        Build the trie using values from ``byte_keys`` iterable.
        """
        if weights is None:
            weights = itertools.repeat(1.0)

        cdef char* data
        cdef float weight
        cdef keyset.Keyset *ks = new keyset.Keyset()

        try:
            for key, weight in izip(byte_keys, weights):
                data = key # cast to char*
                ks.push_back(data, len(key), weight)
            self._trie.build(ks[0], self._config_flags(**options))
        finally:
            del ks

    def __richcmp__(self, other, int op):
        if op == 2:    # ==
            if other is self:
                return True
            elif not isinstance(other, _Trie):
                return False

            return (<_Trie>self)._equals(other)
        elif op == 3:  # !=
            return not (self == other)

        raise TypeError("unorderable types: {0} and {1}".format(
            self.__class__, other.__class__))

    cdef bint _equals(self, _Trie other) nogil:
        cdef int num_keys = self._trie.num_keys()
        cdef base.NodeOrder node_order = self._trie.node_order()
        if (other._trie.num_keys() != num_keys or
            other._trie.node_order() != node_order):
            return False

        cdef agent.Agent ag1, ag2
        ag1.set_query(b"")
        ag2.set_query(b"")
        cdef int i
        cdef key.Key key1, key2
        for i in range(num_keys):
            self._trie.predictive_search(ag1)
            other._trie.predictive_search(ag2)
            key1 = ag1.key()
            key2 = ag2.key()
            if (key1.length() != key2.length() or
                strncmp(key1.ptr(), key2.ptr(), key1.length()) != 0):
                return False
        return True

    def __iter__(self):
        return self.iterkeys()

    def __len__(self):
        return self._trie.num_keys()

    def __contains__(self, unicode key):
        cdef bytes _key = key.encode('utf8')
        return self._contains(_key)

    cdef bint _contains(self, bytes key):
        cdef agent.Agent ag
        ag.set_query(key)
        return self._trie.lookup(ag)

    def read(self, f):
        """
        Read a trie from an open file object.

        Works only with "real" disk-based file objects,
        file-like objects are not supported.
        """
        self._trie.read(f.fileno())
        return self

    def write(self, f):
        """
        Read a trie to an open file object.

        Works only with "real" disk-based file objects,
        file-like objects are not supported.
        """
        self._trie.write(f.fileno())

    def save(self, path):
        """ Save trie to a file. """
        with open(path, 'w') as f:
            self.write(f)

    def load(self, path):
        """ Load trie from a file. """
        with open(path, 'r') as f:
            self.read(f)
        return self

    cpdef bytes tobytes(self) except +:
        """
        Return raw trie content as bytes.
        """
        cdef stringstream stream
        iostream.write((<ostream *> &stream)[0], self._trie[0])
        cdef bytes res = stream.str()
        return res

    cpdef frombytes(self, bytes data) except +:
        """
        Load a trie from bytes ``data``.
        """
        cdef stringstream* stream = new stringstream(data)
        try:
            iostream.read((<istream *> stream)[0], self._trie)
        finally:
            del stream
        return self

    def __reduce__(self): # pickling support
        return self.__class__, tuple(), self.tobytes()

    def __setstate__(self, state): # pickling support
        self.frombytes(state)

    def mmap(self, path):
        """
        Mmap trie to a file; this allows lookups without loading full
        trie to memory.
        """
        import sys
        str_path = path.encode(sys.getfilesystemencoding())
        cdef char* c_path = str_path
        self._trie.mmap(c_path)
        return self

    def iterkeys(self, unicode prefix=""):
        """
        Return an iterator over keys that have a prefix ``prefix``.
        """
        cdef agent.Agent ag
        cdef bytes b_prefix = prefix.encode('utf8')
        ag.set_query(b_prefix)

        while self._trie.predictive_search(ag):
            yield _get_key(ag)

    cpdef list keys(self, unicode prefix=""):
        """
        Return a list with all keys with a prefix ``prefix``.
        """
        # non-generator inlined version of iterkeys()
        cdef list res = []
        cdef bytes b_prefix = prefix.encode('utf8')
        cdef agent.Agent ag
        ag.set_query(b_prefix)

        while self._trie.predictive_search(ag):
            res.append(_get_key(ag))

        return res

    def has_keys_with_prefix(self, unicode prefix=""):
        """
        Returns True if any key in the trie begins with ``prefix``.
        """
        cdef agent.Agent ag
        cdef bytes b_prefix = prefix.encode('utf8')
        ag.set_query(b_prefix)
        return self._trie.predictive_search(ag)


cdef class Trie(_Trie):
    """
    This trie stores unicode keys and assigns an unque ID to each key.
    """

    # key_id method is not in _Trie because it won't work for BytesTrie
    cpdef int key_id(self, unicode key) except -1:
        """
        Return unique auto-generated key index for a ``key``.
        Raises KeyError if key is not in this trie.
        """
        cdef bytes _key = key.encode('utf8')
        cdef int res = self._key_id(_key)
        if res == -1:
            raise KeyError(key)
        return res

    def __getitem__(self, unicode key):
        return self.key_id(key)

    def get(self, key, default=None):
        """
        Return a key id for a given key or ``default`` if the key is not found.
        """
        cdef bytes b_key
        cdef int res

        if isinstance(key, unicode):
            b_key = (<unicode>key).encode('utf8')
        else:
            b_key = key

        res = self._key_id(b_key)
        if res == -1:
            return default
        return res

    cpdef restore_key(self, int index):
        """
        Return a key given its index (obtained by ``key_id`` method).
        """
        cdef agent.Agent ag
        ag.set_query(index)
        try:
            self._trie.reverse_lookup(ag)
        except KeyError:
            raise KeyError(index)
        return _get_key(ag)

    cdef int _key_id(self, char* key):
        cdef bint res
        cdef agent.Agent ag
        ag.set_query(key)
        res = self._trie.lookup(ag)
        if not res:
            return -1
        return ag.key().id()

    def iter_prefixes(self, unicode key):
        """
        Return an iterator of all prefixes of a given key.
        """
        cdef bytes b_key = key.encode('utf8')
        cdef agent.Agent ag
        ag.set_query(b_key)

        while self._trie.common_prefix_search(ag):
            yield _get_key(ag)

    def prefixes(self, unicode key):
        """
        Return a list with all prefixes of a given key.
        """
        # this an inlined version of ``list(self.iter_prefixes(key))``

        cdef list res = []
        cdef bytes b_key = key.encode('utf8')
        cdef agent.Agent ag
        ag.set_query(b_key)

        while self._trie.common_prefix_search(ag):
            res.append(_get_key(ag))
        return res

    def iteritems(self, unicode prefix=""):
        """
        Return an iterator over items that have a prefix ``prefix``.
        """
        cdef bytes b_prefix = prefix.encode('utf8')
        cdef agent.Agent ag
        ag.set_query(b_prefix)

        while self._trie.predictive_search(ag):
            yield _get_key(ag), ag.key().id()

    def items(self, unicode prefix=""):
        # inlined for speed
        cdef list res = []
        cdef bytes b_prefix = prefix.encode('utf8')
        cdef agent.Agent ag
        ag.set_query(b_prefix)

        while self._trie.predictive_search(ag):
            res.append((_get_key(ag), ag.key().id()))

        return res


# This symbol is not allowed in utf8 so it is safe to use
# as a separator between utf8-encoded string and binary payload.
# XXX: b'\xff' value changes sort order for BytesTrie and RecordTrie.
# See https://github.com/kmike/DAWG docs for a description of a similar issue.
cdef bytes _VALUE_SEPARATOR = b'\xff'


cdef class BytesTrie(_Trie):
    """
    This class implements read-only Trie-based
    {unicode -> list of bytes objects} mapping.

    This mapping is implemented by appending binary values to
    utf8-encoded keys and storing the result in MARISA-trie.
    """

    cdef bytes _b_value_separator
    cdef unsigned char _c_value_separator

    def __init__(self, arg=None, bytes value_separator=_VALUE_SEPARATOR, **options):
        """
        ``arg`` must be an iterable of tuples (unicode_key, bytes_payload).
        """
        super(BytesTrie, self).__init__()

        self._b_value_separator = value_separator
        self._c_value_separator = <unsigned char>ord(value_separator)

        byte_keys = (self._raw_key(d[0], d[1]) for d in (arg or []))
        self._build(byte_keys, **options)

    cpdef bytes _raw_key(self, unicode key, bytes payload):
        return key.encode('utf8') + self._b_value_separator + payload

    cdef bint _contains(self, bytes key):
        cdef agent.Agent ag
        cdef bytes _key = key + self._b_value_separator
        ag.set_query(_key)
        return self._trie.predictive_search(ag)

    cpdef list prefixes(self, unicode key):
        """
        Return a list with all prefixes of a given key.
        """

        # XXX: is there a char-walking API in libmarisa?
        # This implementation is suboptimal.

        cdef agent.Agent ag
        cdef list res = []
        cdef int key_len = len(key)
        cdef unicode prefix
        cdef bytes b_prefix
        cdef int ind = 1

        while ind <= key_len:
            prefix = key[:ind]
            b_prefix = prefix.encode('utf8') + self._b_value_separator
            ag.set_query(b_prefix)
            if self._trie.predictive_search(ag):
                res.append(prefix)

            ind += 1

        return res

    def __getitem__(self, key):
        cdef list res = self.get(key)
        if res is None:
            raise KeyError(key)
        return res

    cpdef get(self, key, default=None):
        """
        Return a list of payloads (as byte objects) for a given key
        or ``default`` if the key is not found.
        """
        cdef list res

        if isinstance(key, unicode):
            res = self.get_value(<unicode>key)
        else:
            res = self.b_get_value(key)

        if not res:
            return default
        return res

    cpdef list get_value(self, unicode key):
        """
        Return a list of payloads (as byte objects) for a given unicode key.
        """
        cdef bytes b_key = key.encode('utf8')
        return self.b_get_value(b_key)

    cpdef list b_get_value(self, bytes key):
        """
        Return a list of payloads (as byte objects) for a given utf8-encoded key.
        """
        cdef list res = []
        cdef bytes value
        cdef bytes b_prefix = key + self._b_value_separator
        cdef int prefix_len = len(b_prefix)

        cdef agent.Agent ag
        ag.set_query(b_prefix)

        while self._trie.predictive_search(ag):
            value = ag.key().ptr()[prefix_len:ag.key().length()]
            res.append(value)

        return res

    cpdef list items(self, unicode prefix=""):
        # copied from iteritems for speed
        cdef bytes b_prefix = prefix.encode('utf8')
        cdef bytes value
        cdef unicode key
        cdef unsigned char* raw_key
        cdef list res = []
        cdef int i, value_len

        cdef agent.Agent ag
        ag.set_query(b_prefix)

        while self._trie.predictive_search(ag):
            raw_key = <unsigned char*>ag.key().ptr()

            for i in range(0, ag.key().length()):
                if raw_key[i] == self._c_value_separator:
                    break

            key = raw_key[:i].decode('utf8')
            value = raw_key[i+1:ag.key().length()]

            res.append(
                (key, value)
            )
        return res

    def iteritems(self, unicode prefix=""):
        cdef bytes b_prefix = prefix.encode('utf8')
        cdef bytes value
        cdef unicode key
        cdef unsigned char* raw_key
        cdef int i, value_len

        cdef agent.Agent ag
        ag.set_query(b_prefix)

        while self._trie.predictive_search(ag):
            raw_key = <unsigned char*>ag.key().ptr()

            for i in range(0, ag.key().length()):
                if raw_key[i] == self._c_value_separator:
                    break

            key = raw_key[:i].decode('utf8')
            value = raw_key[i+1:ag.key().length()]

            yield key, value

    cpdef list keys(self, unicode prefix=""):
        # copied from iterkeys for speed
        cdef bytes b_prefix = prefix.encode('utf8')
        cdef unicode key
        cdef unsigned char* raw_key
        cdef list res = []
        cdef int i

        cdef agent.Agent ag
        ag.set_query(b_prefix)

        while self._trie.predictive_search(ag):
            raw_key = <unsigned char*>ag.key().ptr()

            for i in range(0, ag.key().length()):
                if raw_key[i] == self._c_value_separator:
                    key = raw_key[:i].decode('utf8')
                    res.append(key)
                    break
        return res

    def iterkeys(self, unicode prefix=""):
        cdef bytes b_prefix = prefix.encode('utf8')
        cdef unicode key
        cdef unsigned char* raw_key
        cdef int i

        cdef agent.Agent ag
        ag.set_query(b_prefix)

        while self._trie.predictive_search(ag):
            raw_key = <unsigned char*>ag.key().ptr()

            for i in range(0, ag.key().length()):
                if raw_key[i] == self._c_value_separator:
                    yield raw_key[:i].decode('utf8')
                    break


cdef class _UnpackTrie(BytesTrie):

    def __init__(self, arg=None, **options):
        keys = ((d[0], self._pack(d[1])) for d in (arg or []))
        super(_UnpackTrie, self).__init__(keys, **options)

    cdef _unpack(self, bytes value):
        return value

    cdef bytes _pack(self, value):
        return value

    cpdef list b_get_value(self, bytes key):
        cdef list values = BytesTrie.b_get_value(self, key)
        return [self._unpack(val) for val in values]

    cpdef list items(self, unicode prefix=""):
        cdef list items = BytesTrie.items(self, prefix)
        return [(key, self._unpack(val)) for (key, val) in items]

    def iteritems(self, unicode prefix=""):
        return ((key, self._unpack(val)) for key, val in BytesTrie.iteritems(self, prefix))


cdef class RecordTrie(_UnpackTrie):
    """
    This class implements read-only Trie-based
    {unicode -> list of tuples} mapping where all tuples are of the
    same structure and may be packed with the same format string
    using python ``struct`` module from the standard library.

    The payload format must be defined at creation time using ``fmt``
    constructor argument; it has the same meaning as ``fmt`` argument
    for functions from ``struct`` module; take a look at
    http://docs.python.org/library/struct.html#format-strings for the
    specification.

    This mapping is implemented by appending binary values to
    utf8-encoded keys and storing the result in MARISA-trie.
    """
    cdef _struct
    cdef _fmt

    def __init__(self, fmt, arg=None, **options):
        """
        ``arg`` must be an iterable of tuples (unicode_key, data_tuple).
        Data tuples will be converted to bytes with
        ``struct.pack(fmt, *data_tuple)``.

        Take a look at
        http://docs.python.org/library/struct.html#format-strings for the
        format string specification.
        """
        self._fmt = fmt
        self._struct = struct.Struct(str(fmt))
        super(RecordTrie, self).__init__(arg, **options)

    cdef _unpack(self, bytes value):
        return self._struct.unpack(value)

    cdef bytes _pack(self, value):
        return self._struct.pack(*value)

    def __reduce__(self):  # pickling support
        return self.__class__, (self._fmt,), self.tobytes()



#! /usr/bin/env python
import glob
import itertools
from distutils.core import setup
from distutils.extension import Extension

MARISA_FILES = [
    'lib/marisa/*.cc',
    'lib/marisa/grimoire.cc',
    'lib/marisa/grimoire/io/*.cc',
    'lib/marisa/grimoire/trie/*.cc',
    'lib/marisa/grimoire/vector/*.cc',
]

MARISA_FILES = list(itertools.chain(*(glob.glob(path) for path in MARISA_FILES)))

setup(
    name="marisa-trie",
    version="0.7",
    description="Static memory-efficient & fast Trie-like structures for Python (based on marisa-trie C++ library)",
    long_description = open('README.rst').read() + "\n\n" + open('CHANGES.rst').read(),
    author='Mikhail Korobov',
    author_email='kmike84@gmail.com',
    url='https://github.com/kmike/marisa-trie/',

    ext_modules = [
        Extension(
            "marisa_trie",
            sources = glob.glob('src/*.cpp') + MARISA_FILES,
            include_dirs=['lib'],
            language = "c++",
        )
    ],

    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Cython',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Text Processing :: Linguistic',
    ],
)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import pytest
import io
import pickle

import marisa_trie
from .utils import get_random_words, get_random_binary


class TestBytesTrie(object):

    def data(self):
        keys = get_random_words(10000)
        values = [key.encode('cp1251') for key in keys]
        return list(zip(keys, values))

    def test_contains(self):
        data = self.data()
        trie = marisa_trie.BytesTrie(data)

        for key, value in data:
            assert key in trie

        non_key = '2135'
        assert non_key not in trie


    def test_getitem_fuzzy(self):
        data = self.data()
        trie = marisa_trie.BytesTrie(data)

        for key, value in data:
            assert trie[key] == [value]

        with pytest.raises(KeyError):
            trie['2135']

    def test_getitem_multiple(self):
        data = [
            ('foo', b'x'),
            ('fo',  b'y'),
            ('foo', b'a'),
        ]
        trie = marisa_trie.BytesTrie(data)
        assert trie['fo'] == [b'y']
        assert trie['foo'] == [b'a', b'x']

    def test_null_bytes_in_values(self):
        keys = get_random_words(10000)
        values = get_random_binary(10000)

        assert any(b'\x00' in p for p in values)

        data = zip(keys, values)
        trie = marisa_trie.BytesTrie(data)

        for key, value in data:
            assert trie[key] == [value]


    def test_keys(self):
        trie = marisa_trie.BytesTrie([
            ('foo', b'x'),
            ('fo',  b'y'),
            ('foo', b'a'),
        ])

        # FIXME: ordering?
        assert trie.keys() == ['foo', 'foo', 'fo']
        assert trie.keys('f') == ['foo', 'foo', 'fo']
        assert trie.keys('fo') == ['foo', 'foo', 'fo']
        assert trie.keys('foo') == ['foo', 'foo']
        assert trie.keys('food') == []
        assert trie.keys('bar') == []

    def test_iterkeys(self):
        keys = get_random_words(1000)
        values = get_random_binary(1000)

        trie = marisa_trie.BytesTrie(zip(keys, values))
        assert trie.keys() == list(trie.iterkeys())

        for key in keys:
            prefix = key[:5]
            assert trie.keys(prefix) == list(trie.iterkeys(prefix))

    def test_items(self):
        data = [
            ('fo',  b'y'),
            ('foo', b'x'),
            ('foo', b'a'),
        ]
        trie = marisa_trie.BytesTrie(data)
        assert set(trie.items()) == set(data)
        assert set(trie.items('f')) == set(data)
        assert set(trie.items('fo')) == set(data)
        assert set(trie.items('foo')) == set(data[1:])
        assert trie.items('food') == []
        assert trie.items('bar') == []

    def test_iteritems(self):
        keys = get_random_words(1000)
        values = get_random_binary(1000)

        trie = marisa_trie.BytesTrie(zip(keys, values))
        assert trie.items() == list(trie.iteritems())

        for key in keys:
            prefix = key[:5]
            assert trie.items(prefix) == list(trie.iteritems(prefix))

    def test_pickling(self):
        trie = marisa_trie.BytesTrie([
            ('foo', b'foo'),
            ('bar', b'bar'),
        ])
        buf = io.BytesIO()
        pickle.dump(trie, buf)
        buf.seek(0)

        trie2 = pickle.load(buf)
        assert trie2['foo'] == [b'foo']
        assert trie2['bar'] == [b'bar']

    def test_has_keys_with_prefix_empty(self):
        empty_trie = marisa_trie.BytesTrie()
        assert not empty_trie.has_keys_with_prefix('')
        assert not empty_trie.has_keys_with_prefix('ab')


    def test_has_keys_with_prefix(self):
        fruit_trie = marisa_trie.BytesTrie([
            ('apple', b'foo'),
            ('pear', b'bar'),
            ('peach', b'baz'),
        ])
        assert fruit_trie.has_keys_with_prefix('')
        assert fruit_trie.has_keys_with_prefix('a')
        assert fruit_trie.has_keys_with_prefix('pe')
        assert fruit_trie.has_keys_with_prefix('pear')
        assert not fruit_trie.has_keys_with_prefix('x')



class TestRecordTrie(object):

    def data(self):
        keys = get_random_words(10000)
        values = [(len(key), 'ё' in key) for key in keys]
        fmt = str("<H?")
        return fmt, list(zip(keys, values))


    def test_getitem(self):
        fmt, data = self.data()
        trie = marisa_trie.RecordTrie(fmt, data)

        for key, value in data:
            assert trie[key] == [value]

        with pytest.raises(KeyError):
            trie['2135']

    def test_items(self):
        fmt, data = self.data()
        trie = marisa_trie.RecordTrie(fmt, data)

        assert set(trie.items()) == set(data)

    def test_iteritems(self):
        fmt, data = self.data()
        trie = marisa_trie.RecordTrie(fmt, data)
        assert trie.items() == list(trie.iteritems())

        for key, value in data:
            prefix = key[:5]
            assert trie.items(prefix) == list(trie.iteritems(prefix))


    def test_prefixes(self):
        trie = marisa_trie.RecordTrie(str("<H"), [
            ('foo', [1]),
            ('bar',  [2]),
            ('foobar', [3]),
        ])
        assert trie.prefixes('foo') == ['foo']
        assert trie.prefixes('foobar') == ['foo', 'foobar']
        assert trie.prefixes('bara') == ['bar']
        assert trie.prefixes('f') == []

    def test_get(self):
        trie = marisa_trie.RecordTrie(str("<H"), [
            ('foo', [1]),
            ('bar',  [2]),
            ('foobar', [3]),
        ])

        assert trie.get('foo') == [(1,)]
        assert trie.get('foo', [(55,)]) == [(1,)]
        assert trie.get('FOO') is None
        assert trie.get('FOO', [(55,)]) == [(55,)]
        assert trie.get('FOO', 123) == 123

    def test_pickling(self):
        trie = marisa_trie.RecordTrie(str("<H"), [
            ('foo', [1]),
            ('bar', [2]),
        ])
        buf = io.BytesIO()
        pickle.dump(trie, buf)
        buf.seek(0)

        trie2 = pickle.load(buf)
        assert trie2['foo'] == [(1,)]
        assert trie2['bar'] == [(2,)]

